// PhBookView.cpp : implementation of the CPhBookView class
//

#include "stdafx.h"
#include "PhBook.h"

#include "PhBookDoc.h"
#include "PhBookView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPhBookView

IMPLEMENT_DYNCREATE(CPhBookView, CFormView)

BEGIN_MESSAGE_MAP(CPhBookView, CFormView)
	//{{AFX_MSG_MAP(CPhBookView)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_DEL, OnDel)
	ON_BN_CLICKED(IDC_FIND, OnFind)
	ON_LBN_DBLCLK(IDC_LIST, OnDblclkList)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPhBookView construction/destruction

CPhBookView::CPhBookView()
	: CFormView(CPhBookView::IDD)
{
	//{{AFX_DATA_INIT(CPhBookView)
	sName = _T("");
	sPhone = _T("");
	sNotFound = _T("");
	sTotal = _T("");
	//}}AFX_DATA_INIT
}

CPhBookView::~CPhBookView()
{
}

void CPhBookView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPhBookView)
	DDX_Control(pDX, IDC_LIST, cList);
	DDX_Text(pDX, IDC_NAME, sName);
	DDX_Text(pDX, IDC_PHONE, sPhone);
	DDX_Text(pDX, IDC_NFND, sNotFound);
	DDX_Text(pDX, IDC_NUM, sTotal);
	//}}AFX_DATA_MAP
}

BOOL CPhBookView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CFormView::PreCreateWindow(cs);
}

void CPhBookView::OnInitialUpdate()
{
				//   ,    OnUpdate
	CFormView::OnInitialUpdate();
				//       
	cList.ResetContent();
	pDoc = GetDocument();		//  
								//    
	POSITION p = pDoc->cMap.GetStartPosition();
	while (p)					//     ,
	{							//  
		pDoc->cMap.GetNextAssoc(p,sName,sPhone);
								//    List Box
		cList.AddString (MakeEntry());
	}
	cList.SetCurSel(0);			//   
				//      
	OnDblclkList();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();		//  
	pDoc->bFirst = false;
}

/////////////////////////////////////////////////////////////////////////////
// CPhBookView printing

BOOL CPhBookView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CPhBookView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CPhBookView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CPhBookView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CPhBookView diagnostics

#ifdef _DEBUG
void CPhBookView::AssertValid() const
{
	CFormView::AssertValid();
}

void CPhBookView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CPhBookDoc* CPhBookView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPhBookDoc)));
	return (CPhBookDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPhBookView message handlers

void CPhBookView::OnAdd()
{
	UpdateData();					//   
	if (sName.IsEmpty())			//    ,
		return;						// 
	TrimName();						//  
			//       
	if (pDoc->cMap.Lookup (sName,sPhone))
	{		//   ,       
		cList.DeleteString ( cList.FindStringExact(-1,MakeEntry()));
			//       ,
		UpdateData();				//    
		TrimName();					//   
	}
			//       
	int i = cList.AddString (MakeEntry());
	cList.SetTopIndex (i>0 ? i-1 : i);	//  
	cList.SetCurSel (i);			//  
	pDoc->cMap[sName] = sPhone;		//   
	UpdateTexts();					//  
			//  ,   ( )
	pDoc->SetModifiedFlag();
}

void CPhBookView::OnDel()
{
	CString s;
	int i = cList.GetCurSel();		//  
	if (i==LB_ERR)					//  ,
		return;						//  
	cList.GetText (i, s);			//  
	cList.DeleteString (i);			//   
					//     
	sName = s.Left (s.Find (':'));
	TrimName();
	pDoc->cMap.RemoveKey (sName);	//   
	int last = cList.GetCount()-1;	//   
	i = i<=last ? i : i-1;			//  
	cList.SetCurSel (i);			//  i- 
	UpdateTexts();					//  
	pDoc->SetModifiedFlag();		//  
}

void CPhBookView::OnFind()
{
	int i = -1;						//   
	UpdateData();					//    
	if (sName.IsEmpty())			//   ,
		return;						// 
	TrimName();						//  
	sNotFound = "";					//   
									//    ,  ,
	if (pDoc->cMap.Lookup (sName, sPhone))
									//      
		i = cList.FindStringExact (-1, MakeEntry());
									//   ,   
	else if ((i=cList.SelectString (-1,sName)) == LB_ERR)	//    
		sNotFound = " ";	//   
	if (i>=0)						//  
	{								//     
		cList.SetTopIndex (i>0 ? i-1 : i);
		cList.SetCurSel(i);
	}
	UpdateData(FALSE);				//     
}

CString CPhBookView::MakeEntry()
{
	UINT len = sName.GetLength();		//  
	CString pad (' ',40-len);			//   
						//    List Box
	return len >= 40 ? sName.Left(39)+" :  "+sPhone
					: sName+pad+":  "+sPhone;
}

void CPhBookView::UpdateTexts()
{
	sNotFound = "";			//  
		//     
	pDoc->nTotal = pDoc->cMap.GetCount();
		//    
	sTotal.Format("%d",pDoc->nTotal);
	UpdateData(FALSE);		//   
}

void CPhBookView::OnDblclkList()
{
	int i = cList.GetCurSel();	//  
	if (i==LB_ERR)				//     ,
		return;					//  
	CString s;
	cList.GetText(i,s);			//     s
				//     
	sName = s.Left (s.Find (':'));
	TrimName();
	pDoc->cMap.Lookup (sName, sPhone);	//   
	UpdateTexts();
	UpdateData (FALSE);			//    
}

void CPhBookView::TrimName()
{
	int i, cb=0;					// cb - c 
	int len = sName.GetLength();	//  
			// ,    .  ,
	bool white = true;				//   
	TCHAR c;						//  
	CString buf (' ',len);			// ,   
	for (i=0; i<len; i++)			//   
		if ((c=sName[i]) != ' ')	//  
		{							//   whitespace,
			buf.SetAt (cb++,c);		//   
			white = false;			// :   
		}
		else if (cb && !white)		//    
		{							//   
			buf.SetAt (cb++,TCHAR(' '));
			white = true;			// :  
		}
	buf.TrimRight();				//  
	sName = buf;					//    
}

void CPhBookView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	sName="";				//  
	sPhone="";
	sNotFound="";
	UpdateData(FALSE);		//  
}
