// TListView.cpp : implementation of the CTListView class
//

#include "stdafx.h"
#include "TList.h"

#include "TListDoc.h"
#include "TListView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTListView

IMPLEMENT_DYNCREATE(CTListView, CListView)

BEGIN_MESSAGE_MAP(CTListView, CListView)
	//{{AFX_MSG_MAP(CTListView)
	ON_COMMAND(ID_EDIT_CUR, OnEditCurrent)
	ON_COMMAND(ID_EDIT_CUT, OnDelete)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	ON_COMMAND(ID_EDIT_RESTORE, OnEditRestore)
	ON_COMMAND(ID_EDIT_NEW, OnEditNew)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_WM_LBUTTONDOWN()
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUR, OnUpdateEditCmd)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCmd)
	ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditCmd)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CListView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTListView construction/destruction

CTListView::CTListView(): cList(GetListCtrl()) { }

CTListView::~CTListView()
{
}

BOOL CTListView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_SHOWSELALWAYS | LVS_REPORT | LVS_SINGLESEL;
	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTListView drawing

void CTListView::OnDraw(CDC* pDC)
{
	CTListDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

void CTListView::OnInitialUpdate()
{									//  
	CListView::OnInitialUpdate();	//  
	sFind = _T("");					//   
	InitList();						// Helper-function
}

/////////////////////////////////////////////////////////////////////////////
// CTListView printing

BOOL CTListView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTListView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CTListView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CTListView diagnostics

#ifdef _DEBUG
void CTListView::AssertValid() const
{
	CListView::AssertValid();
}

void CTListView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CTListDoc* CTListView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTListDoc)));
	return (CTListDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTListView message handlers

void CTListView::InitList()
{		//    
		//      (. Help)
	LV_COLUMN lvc = { LVCF_FMT, LVCFMT_LEFT,0,NULL,0,0 };
							//     ,
	if (!cList.GetColumn (0,&lvc))
	{						//   
		cList.InsertColumn (0,"",LVCFMT_LEFT,210);
		cList.InsertColumn (1,"",LVCFMT_LEFT,70);
		cList.InsertColumn (2,"",LVCFMT_LEFT,250);
	}
	pDoc = GetDocument();			//  
		//        CEntry
	CEntry *p;
		//      ( )
	iNum = pDoc->cMap.GetCount();
		//    
	POSITION pos = pDoc->cMap.GetStartPosition();

	for (int i=0;  i<iNum;  i++)		//  
	{				//       
		pDoc->cMap.GetNextAssoc (pos,sName,p);
					//      
		cList.InsertItem (LVIF_TEXT,i,sName,0,0,0,0L);
					//     .   .
		cList.SetItemText (i,0,sName);
					//       CEntry
		cList.SetItemText (i,1,p->sPhone);
		cList.SetItemText (i,2,p->sAddr);
	}
					//   
	SelectItem();	// Helper-function (. )
}

void CTListView::SelectItem()
{
	pDoc = GetDocument();				//  
	iNum = pDoc->cMap.GetCount();		//  
	if ( !iNum )						//   ,
		return;							//  
				//   ,  ,   
	int sel = cList.GetNextItem (-1,LVNI_SELECTED);
	int foc = cList.GetNextItem (-1,LVNI_FOCUSED);
				//     - 
	if (iSel < 0 || iNum-1 < iSel)
		iSel = sel>=0 ? sel : foc>=0 ? foc : 0;
				//      
	cList.SetItem (iSel,0,LVIF_STATE,0,0,
								LVIS_SELECTED,LVIS_SELECTED,0);
	cList.SetItem (iSel,0,LVIF_STATE,0,0,
								LVIS_FOCUSED,LVIS_FOCUSED,0);
				//      
	cList.EnsureVisible (iSel,false);
}

void CTListView::AddItem()
{		//  
	TrimName();							//  
		//          
	LV_FINDINFO FindInfo = { LVFI_STRING,sName,0L };
		//  . 
		//     ,       
	iSel = cList.FindItem (&FindInfo,-1);
		//     ,  
	if (iSel>=0 && !SafeDelete())
		return;	
		//     CEntry    
	CEntry *pEntry = new CEntry(sPhone,sAddr);
		//        
	iSel=0;
	cList.InsertItem (LVIF_TEXT,iSel,sName,0,0,0,0L);
	cList.SetItemText (iSel,1,sPhone);
	cList.SetItemText (iSel,2,sAddr);
								//    ( )
	pDoc->cMap[sName]=pEntry;
	SelectItem ();				//   
	pDoc->SetModifiedFlag();	//   
}

void CTListView::TrimName()
{
	int i, cb=0;					// cb - c 
	int len = sName.GetLength();	//  
			// ,    .  ,
	bool white = true;				//   
	TCHAR c;						//  
	CString buf (' ',len);			// ,   
	for (i=0; i<len; i++)			//   
		if ((c=sName[i]) != ' ')	//  
		{							//   whitespace,
			buf.SetAt (cb++,c);		//   
			white = false;			// :   
		}
		else if (cb && !white)		//    
		{							//   
			buf.SetAt (cb++,TCHAR(' '));
			white = true;			// :  
		}
	buf.TrimRight();				//  
	sName = buf;					//    
}

bool CTListView::SafeDelete()
{										//   
	SelectItem();						//   
		//   ,     
	CString s = " :  " +	cList.GetItemText (iSel,0)	
						+ "   "		+			cList.GetItemText (iSel,1)
						+ "   " 	+			cList.GetItemText (iSel,2)
						+ " ?";
										//    
	if (AfxMessageBox (s,MB_YESNO | MB_ICONQUESTION)==IDNO)
		return false;					//  
	DeleteIt();							//  
	return true;						//  
}

void CTListView::DeleteIt()
{
	sName = cList.GetItemText (iSel,0);		//   
	pDoc->cMap.RemoveKey (sName);				//   
	pDoc->SetModifiedFlag();						//  
	cList.DeleteItem (iSel);						//   
}


void CTListView::OnEditCurrent()
{
	CEditDlg dlg;				//    
	SelectItem();				//   
								//   
	sName = cList.GetItemText (iSel,0);
								//     
	dlg.sPhone = sPhone = cList.GetItemText (iSel,1);
	dlg.sAddr = sAddr = cList.GetItemText (iSel,2);
	if (dlg.DoModal()!=IDOK)	//  
		return;
	sPhone = dlg.sPhone;		//   
	sAddr  = dlg.sAddr;
								//    
	cList.SetItemText (iSel,1,sPhone);
	cList.SetItemText (iSel,2,sAddr);
	pDoc = GetDocument();		//     
	CEntry *p;					//     
								//   
	if ( ! pDoc->cMap.Lookup(sName,p) )
		return;					//   , ..    
								//      
	p->sPhone = sPhone;
	p->sAddr = sAddr;
	pDoc->SetModifiedFlag();	//  
}

void CTListView::OnDelete()
{
	if (iSel>=0)				//   ,
		DeleteIt();				//   
	else						//  ,
		if (!SafeDelete())		//   
			return;
								//    ,
	if (cList.GetNextItem (iSel,LVNI_BELOW)<0)
		iSel--;					//   ,   
	SelectItem();
}

void CTListView::OnEditFind()
{
	CFindDlg dlg(this);			//    
								//      
	dlg.sFind = sFind;
	if (dlg.DoModal()!=IDOK)	//   
		return;					// 
	sFind = dlg.sFind;			//    
				//    ,
				//       CListCtrl
	LV_FINDINFO FindInfo = { LVFI_PARTIAL,sFind,0L };
				//     
	int id = cList.FindItem (&FindInfo,-1);

	if (id<0)	//    ,    
		AfxMessageBox (sFind + "    ");
	else
	{					//   ,
		iSel = id;		//    
		SelectItem();	//    
	}
}

void CTListView::OnEditRestore()
{
				//       
	cList.DeleteAllItems();
				//    
	GetDocument()->Restore();
				//    
	InitList();
}

void CTListView::OnEditNew()
{
	sName  = "";		//      
	sPhone = "";		//   
	sAddr  = "";
	CNewDlg(this).DoModal();	//  
}


void CTListView::OnColumnclick (NMHDR*, LRESULT*)
{
	cList.SortItems (NULL,0);		//    
									//   
	iSel = cList.GetNextItem (0, LVNI_SELECTED);
}

void CTListView::OnLButtonDown (UINT nFlags, CPoint point)
{					//      
	iSel = cList.HitTest (point);
	CListView::OnLButtonDown (nFlags, point);
}

void CTListView::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (iSel < 0)		//    , 
		return;			//  
	OnEditCurrent();	//      
	*pResult = 0;		//   

}

void CTListView::OnUpdateEditCmd(CCmdUI* pCmdUI)
{
		//  ,    
	pCmdUI->Enable(cList.GetItemCount());
}
/////////////////////////////////////////////////////////////////////////////
// CEditDlg dialog


CEditDlg::CEditDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEditDlg)
	sPhone = _T("");
	sAddr = _T("");
	//}}AFX_DATA_INIT
}


void CEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditDlg)
	DDX_Text(pDX, IDC_PHONE, sPhone);
	DDX_Text(pDX, IDC_ADDR, sAddr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEditDlg, CDialog)
	//{{AFX_MSG_MAP(CEditDlg)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEditDlg message handlers
/////////////////////////////////////////////////////////////////////////////
// CFindDlg dialog


CFindDlg::CFindDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFindDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFindDlg)
	sFind = _T("");
	//}}AFX_DATA_INIT
}


void CFindDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFindDlg)
	DDX_Text(pDX, IDC_FIND, sFind);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFindDlg, CDialog)
	//{{AFX_MSG_MAP(CFindDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFindDlg message handlers
/////////////////////////////////////////////////////////////////////////////
// CNewDlg dialog


CNewDlg::CNewDlg(CTListView* p)		// 
		: CDialog(CNewDlg::IDD, NULL)
{
	pView = p;						//    
	//{{AFX_DATA_INIT(CNewDlg)		//  ClassWizard
	//}}AFX_DATA_INIT
}


void CNewDlg::DoDataExchange(CDataExchange* pDX)
{									//  
	CDialog::DoDataExchange(pDX);	//   
	//{{AFX_DATA_MAP(CNewDlg)
	//}}AFX_DATA_MAP
					//      ClassWizard
					//        .
	DDX_Text(pDX, IDC_NAME, pView->sName);
	DDX_Text(pDX, IDC_PHONE, pView->sPhone);
	DDX_Text(pDX, IDC_ADDR, pView->sAddr);
}


BEGIN_MESSAGE_MAP(CNewDlg, CDialog)
	//{{AFX_MSG_MAP(CNewDlg)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNewDlg message handlers

void CNewDlg::OnAdd()
{
	UpdateData();							//    
	if (pView->sName.IsEmpty())	//    ,
		return;									//  
							//  ,     
	pView->AddItem();
}

